@@ -17,9 +17,7 @@ pub use libsignal_bridge_types::{
 #[cfg(feature = "ffi")]
 pub use libsignal_bridge_types::{ffi, ffi_arg_type, ffi_result_type};
 #[cfg(feature = "jni")]
-pub use libsignal_bridge_types::{
-    jni, jni_arg_type, jni_args, jni_class_name, jni_result_type, jni_signature,
-};
+pub use libsignal_bridge_types::{jni, jni_arg_type, jni_args, jni_result_type, jni_signature};
 #[cfg(feature = "node")]
 pub use libsignal_bridge_types::{node, node_register};
 
