@@ -9,7 +9,7 @@ use std::sync::Arc;
 
 use itertools::Itertools as _;
 use jni::JNIEnv;
-use jni::objects::{AutoLocal, JByteBuffer, JMap, JObjectArray};
+use jni::objects::{AutoLocal, JByteBuffer, JIntArray, JMap, JObjectArray};
 use jni::sys::{JNI_FALSE, JNI_TRUE, jbyte};
 use libsignal_account_keys::{AccountEntropyPool, InvalidAccountEntropyPool};
 use libsignal_core::try_scoped;
@@ -20,9 +20,10 @@ use paste::paste;
 use super::*;
 use crate::io::{InputStream, SyncInputStream};
 use crate::message_backup::MessageBackupValidationOutcome;
-use crate::net::chat::ChatListener;
+use crate::net::chat::{
+    ChatListener, JniChatListener, JniProvisioningListener, ProvisioningListener,
+};
 use crate::net::registration::{ConnectChatBridge, RegistrationPushToken};
-use crate::protocol::KyberPublicKey;
 use crate::support::{Array, AsType, FixedLengthBincodeSerializable, Serialized, extend_lifetime};
 
 /// Converts arguments from their JNI form to their Rust form.
@@ -120,9 +121,40 @@ where
     }
 }
 
+/// A variation of [`ArgTypeInfo`] for callback results.
+///
+/// All [`SimpleArgTypeInfo`] implementations are reusable for this, but the general [`ArgTypeInfo`]
+/// allows borrowing from the foreign value and a callback result can't do that.
+///
+/// Note that because a JNI type signature is required, while [`SimpleArgTypeInfo`] doesn't provide
+/// one, the blanket implementation for `CallbackResultTypeInfo` also requires [`ResultTypeInfo`].
+/// If this is not available, you'll have to provide a manual implementation of
+/// `CallbackResultTypeInfo` instead.
+pub trait CallbackResultTypeInfo<'a>: Sized {
+    type ResultType;
+    const JNI_RESULT_SIGNATURE: &'static str;
+    fn convert_from_callback(
+        env: &mut JNIEnv<'a>,
+        foreign: Self::ResultType,
+    ) -> Result<Self, BridgeLayerError>;
+}
+
+impl<'a, T: SimpleArgTypeInfo<'a> + ResultTypeInfo<'a>> CallbackResultTypeInfo<'a> for T {
+    type ResultType = <T as SimpleArgTypeInfo<'a>>::ArgType;
+    const JNI_RESULT_SIGNATURE: &'static str = <T as ResultTypeInfo<'a>>::JNI_SIGNATURE;
+
+    fn convert_from_callback(
+        env: &mut JNIEnv<'a>,
+        foreign: Self::ResultType,
+    ) -> Result<Self, BridgeLayerError> {
+        Self::convert_from(env, &foreign)
+    }
+}
+
 /// Converts result values from their Rust form to their FFI form.
 ///
 /// `ResultTypeInfo` is used to implement the `bridge_fn` macro, but can also be used outside it.
+/// `ResultTypeInfo` is also used for callback arguments in the `bridge_callback` macro.
 ///
 /// ```no_run
 /// # use libsignal_bridge_types::jni::*;
@@ -144,6 +176,17 @@ where
 pub trait ResultTypeInfo<'a>: Sized {
     /// The JNI form of the result (e.g. `jint`).
     type ResultType: Into<JValueOwned<'a>>;
+
+    /// The JNI signature of the result type.
+    ///
+    /// Use [`jni_signature`] to fill this in properly. In particular, for class types this should
+    /// be a *specific* class, not just "Object".
+    ///
+    /// Defaulted to empty because most types aren't used in a context where the signature is
+    /// needed. To be sure a type is providing a signature, use [`jni_signature_for`] instead of
+    /// referencing this constant directly.
+    const JNI_SIGNATURE: &'static str = "";
+
     /// Converts the data in `self` to the JNI type, similar to `try_into()`.
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError>;
 }
@@ -316,6 +359,30 @@ impl<'a> SimpleArgTypeInfo<'a> for AccountEntropyPool {
     }
 }
 
+impl<'a> SimpleArgTypeInfo<'a>
+    for libsignal_net_chat::api::messages::MultiRecipientSendAuthorization
+{
+    type ArgType = JByteArray<'a>;
+    fn convert_from(
+        env: &mut JNIEnv<'a>,
+        foreign: &Self::ArgType,
+    ) -> Result<Self, BridgeLayerError> {
+        // If we ever have more than two options, we won't be able to just use null for one of them,
+        // but for now this is convenient.
+        if foreign.is_null() {
+            Ok(Self::Story)
+        } else {
+            let mut elements_guard = <&[u8]>::borrow(env, foreign)?;
+            let bytes = <&[u8]>::load_from(&mut elements_guard);
+            let token =
+                zkgroup::deserialize(bytes).map_err(|_: ZkGroupDeserializationFailure| {
+                    BridgeLayerError::BadArgument("bad GroupSendFullToken".into())
+                })?;
+            Ok(Self::Group(token))
+        }
+    }
+}
+
 impl<'a> SimpleArgTypeInfo<'a> for Box<[u8]> {
     type ArgType = JByteArray<'a>;
 
@@ -584,7 +651,7 @@ macro_rules! bridge_trait {
 }
 
 bridge_trait!(IdentityKeyStore);
-bridge_trait!(PreKeyStore);
+// bridge_trait!(PreKeyStore);
 bridge_trait!(SenderKeyStore);
 bridge_trait!(SessionStore);
 bridge_trait!(SignedPreKeyStore);
@@ -592,11 +659,52 @@ bridge_trait!(KyberPreKeyStore);
 bridge_trait!(InputStream);
 bridge_trait!(SyncInputStream);
 
+impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param, 'context>
+    for &'storage mut dyn PreKeyStore
+{
+    type ArgType = JObject<'context>;
+    type StoredType = BridgedStore<JniBridgePreKeyStore>;
+    fn borrow(
+        env: &mut JNIEnv<'context>,
+        store: &'param Self::ArgType,
+    ) -> Result<Self::StoredType, BridgeLayerError> {
+        Ok(BridgedStore(JniBridgePreKeyStore::new(env, store)?))
+    }
+    fn load_from(stored: &'storage mut Self::StoredType) -> Self {
+        stored
+    }
+}
+
+impl<'a> CallbackResultTypeInfo<'a> for Option<PreKeyRecord> {
+    type ResultType = JObject<'a>;
+    const JNI_RESULT_SIGNATURE: &'static str =
+        jni_signature!(org.signal.libsignal.internal.NativeHandleGuard::Owner);
+
+    fn convert_from_callback(
+        env: &mut JNIEnv<'a>,
+        foreign: Self::ResultType,
+    ) -> Result<Self, BridgeLayerError> {
+        if foreign.is_null() {
+            Ok(None)
+        } else {
+            let handle: jlong = call_method_checked(
+                env,
+                foreign,
+                "unsafeNativeHandleWithoutGuard",
+                jni_args!(() -> long),
+            )?;
+            let object: &PreKeyRecord =
+                unsafe { BridgeHandle::native_handle_cast(handle)?.as_ref() };
+            Ok(Some(object.clone()))
+        }
+    }
+}
+
 impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param, 'context>
     for Option<Box<dyn ChatListener>>
 {
     type ArgType = JObject<'context>;
-    type StoredType = Option<JniBridgeChatListener>;
+    type StoredType = Option<JniChatListener>;
     fn borrow(
         env: &mut JNIEnv<'context>,
         store: &'param Self::ArgType,
@@ -604,11 +712,11 @@ impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param,
         if store.is_null() {
             Ok(None)
         } else {
-            Ok(Some(JniBridgeChatListener::new(env, store)?))
+            Ok(Some(JniChatListener::new(env, store)?))
         }
     }
     fn load_from(stored: &'storage mut Self::StoredType) -> Self {
-        stored.take().map(|j| j.into_listener())
+        stored.take().map(|j| Box::new(j) as Box<_>)
     }
 }
 
@@ -616,7 +724,7 @@ impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param,
     for Box<dyn ChatListener>
 {
     type ArgType = JObject<'context>;
-    type StoredType = Option<JniBridgeChatListener>;
+    type StoredType = Option<JniChatListener>;
     fn borrow(
         env: &mut JNIEnv<'context>,
         store: &'param Self::ArgType,
@@ -624,10 +732,31 @@ impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param,
         if store.is_null() {
             return Err(BridgeLayerError::NullPointer(Some("BridgeChatListener")));
         }
-        Ok(Some(JniBridgeChatListener::new(env, store)?))
+        Ok(Some(JniChatListener::new(env, store)?))
     }
     fn load_from(stored: &'storage mut Self::StoredType) -> Self {
-        stored.take().expect("not previously taken").into_listener()
+        Box::new(stored.take().expect("not previously taken"))
+    }
+}
+
+impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param, 'context>
+    for Box<dyn ProvisioningListener>
+{
+    type ArgType = JObject<'context>;
+    type StoredType = Option<JniProvisioningListener>;
+    fn borrow(
+        env: &mut JNIEnv<'context>,
+        store: &'param Self::ArgType,
+    ) -> Result<Self::StoredType, BridgeLayerError> {
+        if store.is_null() {
+            return Err(BridgeLayerError::NullPointer(Some(
+                "BridgeProvisioningListener",
+            )));
+        }
+        Ok(Some(JniProvisioningListener::new(env, store)?))
+    }
+    fn load_from(stored: &'storage mut Self::StoredType) -> Self {
+        Box::new(stored.take().expect("not previously taken"))
     }
 }
 
@@ -651,66 +780,42 @@ impl<'storage, 'param: 'storage, 'context: 'param> ArgTypeInfo<'storage, 'param,
 impl<'a> SimpleArgTypeInfo<'a> for CiphertextMessageRef<'a> {
     type ArgType = JavaCiphertextMessage<'a>;
     fn convert_from(env: &mut JNIEnv, foreign: &Self::ArgType) -> Result<Self, BridgeLayerError> {
-        fn native_handle_from_message<'a, T: BridgeHandle>(
-            env: &mut JNIEnv,
-            foreign: &JavaCiphertextMessage<'a>,
-            class_name: &'static str,
-            make_result: fn(&'a T) -> CiphertextMessageRef<'a>,
-        ) -> Result<Option<CiphertextMessageRef<'a>>, BridgeLayerError> {
-            if env
-                .is_instance_of(foreign, class_name)
-                .check_exceptions(env, "CiphertextMessageRef::convert_from")?
-            {
-                let handle: jlong = call_method_checked(
-                    env,
-                    foreign,
-                    "unsafeNativeHandleWithoutGuard",
-                    jni_args!(() -> long),
-                )?;
-                Ok(Some(make_result(unsafe {
-                    T::native_handle_cast(handle)?.as_ref()
-                })))
-            } else {
-                Ok(None)
-            }
-        }
-
         if foreign.is_null() {
             return Err(BridgeLayerError::NullPointer(Some("CipherTextMessageRef")));
         }
 
         None.or_else(|| {
-            native_handle_from_message(
+            map_native_handle_if_matching_jobject(
                 env,
                 foreign,
-                jni_class_name!(org.signal.libsignal.protocol.message.SignalMessage),
+                ClassName("org.signal.libsignal.protocol.message.SignalMessage"),
                 Self::SignalMessage,
             )
             .transpose()
         })
         .or_else(|| {
-            native_handle_from_message(
+            map_native_handle_if_matching_jobject(
                 env,
                 foreign,
-                jni_class_name!(org.signal.libsignal.protocol.message.PreKeySignalMessage),
+                ClassName("org.signal.libsignal.protocol.message.PreKeySignalMessage"),
                 Self::PreKeySignalMessage,
             )
             .transpose()
         })
         .or_else(|| {
-            native_handle_from_message(
+            map_native_handle_if_matching_jobject(
                 env,
                 foreign,
-                jni_class_name!(org.signal.libsignal.protocol.message.SenderKeyMessage),
+                ClassName("org.signal.libsignal.protocol.message.SenderKeyMessage"),
                 Self::SenderKeyMessage,
             )
             .transpose()
         })
         .or_else(|| {
-            native_handle_from_message(
+            map_native_handle_if_matching_jobject(
                 env,
                 foreign,
-                jni_class_name!(org.signal.libsignal.protocol.message.PlaintextContent),
+                ClassName("org.signal.libsignal.protocol.message.PlaintextContent"),
                 Self::PlaintextContent,
             )
             .transpose()
@@ -741,6 +846,26 @@ impl<'a> ResultTypeInfo<'a> for crate::cds2::Cds2Metrics {
     }
 }
 
+impl ResultTypeInfo<'_> for () {
+    type ResultType = ();
+    const JNI_SIGNATURE: &'static str = jni_signature!(void);
+    fn convert_into(self, _env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
+        Ok(self)
+    }
+}
+
+impl CallbackResultTypeInfo<'_> for () {
+    type ResultType = ();
+    const JNI_RESULT_SIGNATURE: &'static str = jni_signature!(void);
+
+    fn convert_from_callback(
+        _env: &mut JNIEnv<'_>,
+        _foreign: Self::ResultType,
+    ) -> Result<Self, BridgeLayerError> {
+        Ok(())
+    }
+}
+
 impl ResultTypeInfo<'_> for bool {
     type ResultType = jboolean;
     fn convert_into(self, _env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
@@ -769,6 +894,7 @@ impl ResultTypeInfo<'_> for u16 {
 /// Note that this is different from the implementation of [`ArgTypeInfo`] for `u32`.
 impl ResultTypeInfo<'_> for u32 {
     type ResultType = jint;
+    const JNI_SIGNATURE: &'static str = jni_signature!(int);
     fn convert_into(self, _env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
         // Note that we don't check bounds here.
         Ok(self as jint)
@@ -800,6 +926,7 @@ impl ResultTypeInfo<'_> for u64 {
 /// Note that this is different from the implementation of [`ArgTypeInfo`] for `Timestamp`.
 impl ResultTypeInfo<'_> for crate::protocol::Timestamp {
     type ResultType = jlong;
+    const JNI_SIGNATURE: &'static str = jni_signature!(long);
     fn convert_into(self, _env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
         // Note that we don't check bounds here.
         Ok(self.epoch_millis() as jlong)
@@ -830,6 +957,7 @@ impl ResultTypeInfo<'_> for crate::zkgroup::Timestamp {
 
 impl<'a> ResultTypeInfo<'a> for String {
     type ResultType = JString<'a>;
+    const JNI_SIGNATURE: &'static str = jni_signature!(java.lang.String);
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
         self.deref().convert_into(env)
     }
@@ -862,6 +990,7 @@ impl<'a> ResultTypeInfo<'a> for Option<&str> {
 
 impl<'a> ResultTypeInfo<'a> for &[u8] {
     type ResultType = JByteArray<'a>;
+    const JNI_SIGNATURE: &'static str = jni_signature!([byte]);
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
         env.byte_array_from_slice(self)
             .check_exceptions(env, "<&[u8]>::convert_into")
@@ -870,6 +999,7 @@ impl<'a> ResultTypeInfo<'a> for &[u8] {
 
 impl<'a> ResultTypeInfo<'a> for Option<&[u8]> {
     type ResultType = JByteArray<'a>;
+    const JNI_SIGNATURE: &'static str = <&'static [u8]>::JNI_SIGNATURE;
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
         match self {
             Some(s) => s.convert_into(env),
@@ -880,6 +1010,15 @@ impl<'a> ResultTypeInfo<'a> for Option<&[u8]> {
 
 impl<'a> ResultTypeInfo<'a> for Vec<u8> {
     type ResultType = JByteArray<'a>;
+    const JNI_SIGNATURE: &'static str = <&'static [u8]>::JNI_SIGNATURE;
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        self.deref().convert_into(env)
+    }
+}
+
+impl<'a> ResultTypeInfo<'a> for bytes::Bytes {
+    type ResultType = JByteArray<'a>;
+    const JNI_SIGNATURE: &'static str = <&'static [u8]>::JNI_SIGNATURE;
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
         self.deref().convert_into(env)
     }
@@ -1149,6 +1288,7 @@ where
 
 impl<T: BridgeHandle> ResultTypeInfo<'_> for T {
     type ResultType = ObjectHandle;
+    const JNI_SIGNATURE: &'static str = jni_signature!(long);
     fn convert_into(self, _env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
         Ok(T::encode_as_handle(Arc::new(self)))
     }
@@ -1156,6 +1296,7 @@ impl<T: BridgeHandle> ResultTypeInfo<'_> for T {
 
 impl<T: BridgeHandle> ResultTypeInfo<'_> for Option<T> {
     type ResultType = ObjectHandle;
+    const JNI_SIGNATURE: &'static str = T::JNI_SIGNATURE;
     fn convert_into(self, env: &mut JNIEnv) -> Result<Self::ResultType, BridgeLayerError> {
         match self {
             Some(obj) => obj.convert_into(env),
@@ -1165,17 +1306,28 @@ impl<T: BridgeHandle> ResultTypeInfo<'_> for Option<T> {
 }
 
 impl<'a, A: ResultTypeInfo<'a>, B: ResultTypeInfo<'a>> ResultTypeInfo<'a> for (A, B) {
-    type ResultType = JavaPair<'a>;
+    type ResultType = JavaPair<'a, A::ResultType, B::ResultType>;
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
         let a = self.0.convert_into(env)?;
         let a = box_primitive_if_needed(env, a.into())?;
         let b = self.1.convert_into(env)?;
         let b = box_primitive_if_needed(env, b.into())?;
-        new_instance(
+        Ok(new_instance(
             env,
-            ClassName("org.signal.libsignal.protocol.util.Pair"),
+            ClassName("kotlin.Pair"),
             jni_args!((a => java.lang.Object, b => java.lang.Object) -> void),
-        )
+        )?
+        .into())
+    }
+}
+
+impl<'a, A: ResultTypeInfo<'a>, B: ResultTypeInfo<'a>> ResultTypeInfo<'a> for Option<(A, B)> {
+    type ResultType = JavaPair<'a, A::ResultType, B::ResultType>;
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        let Some(value) = self else {
+            return Ok(JObject::null().into());
+        };
+        value.convert_into(env)
     }
 }
 
@@ -1445,34 +1597,29 @@ impl<'a> SimpleArgTypeInfo<'a> for crate::net::registration::SignedPublicPreKey
         })?;
 
         let public_key = {
-            let native_handle = call_method_checked(
-                env,
-                &public_key,
-                "unsafeNativeHandleWithoutGuard",
-                jni_args!(() -> long),
-            )?;
-
-            if env
-                .is_instance_of(
+            None.or_else(|| {
+                map_native_handle_if_matching_jobject(
+                    env,
                     &public_key,
-                    jni_class_name!(org.signal.libsignal.protocol.ecc.ECPublicKey),
+                    ClassName("org.signal.libsignal.protocol.ecc.ECPublicKey"),
+                    PublicKey::serialize,
                 )
-                .expect_no_exceptions()?
-            {
-                unsafe { PublicKey::native_handle_cast(native_handle)?.as_ref() }.serialize()
-            } else if env
-                .is_instance_of(
+                .transpose()
+            })
+            .or_else(|| {
+                map_native_handle_if_matching_jobject(
+                    env,
                     &public_key,
-                    jni_class_name!(org.signal.libsignal.protocol.kem.KEMPublicKey),
+                    ClassName("org.signal.libsignal.protocol.kem.KEMPublicKey"),
+                    kem::PublicKey::serialize,
                 )
-                .expect_no_exceptions()?
-            {
-                unsafe { KyberPublicKey::native_handle_cast(native_handle)?.as_ref() }.serialize()
-            } else {
-                return Err(BridgeLayerError::BadArgument(
+                .transpose()
+            })
+            .unwrap_or_else(|| {
+                Err(BridgeLayerError::BadArgument(
                     "publicKey type is not supported".to_owned(),
-                ));
-            }
+                ))
+            })?
         };
         let signature = <Box<[u8]>>::convert_from(env, &signature)?;
 
@@ -1650,11 +1797,9 @@ impl<'a> ResultTypeInfo<'a> for &'_ [libsignal_net_chat::api::ChallengeOption] {
     type ResultType = JObjectArray<'a>;
 
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
-        make_object_array(
-            env,
-            jni_class_name!(org.signal.libsignal.net.ChallengeOption),
-            self.iter().copied(),
-        )
+        let element_class = find_class(env, ClassName("org.signal.libsignal.net.ChallengeOption"))
+            .check_exceptions(env, "ChallengeOption::convert_into")?;
+        make_object_array(env, element_class, self.iter().copied())
     }
 }
 
@@ -1666,6 +1811,83 @@ impl<'a> ResultTypeInfo<'a> for Box<[libsignal_net_chat::api::ChallengeOption]>
     }
 }
 
+impl<'a> ResultTypeInfo<'a> for Vec<ServiceId> {
+    type ResultType = JObjectArray<'a>;
+
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        let element_class = find_primitive_array_class(env, jni_signature!([byte]))
+            .check_exceptions(env, "Vec<ServiceId>::convert_into")?;
+        make_object_array(env, element_class, self)
+    }
+}
+
+impl<'a> ResultTypeInfo<'a> for &'_ libsignal_net_chat::api::messages::MismatchedDeviceError {
+    type ResultType = JObject<'a>;
+
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        let libsignal_net_chat::api::messages::MismatchedDeviceError {
+            account,
+            missing_devices,
+            extra_devices,
+            stale_devices,
+        } = self;
+
+        fn convert_device_list<'a>(
+            env: &mut JNIEnv<'a>,
+            input: &[DeviceId],
+        ) -> Result<JIntArray<'a>, BridgeLayerError> {
+            let len = input.len().try_into().map_err(|_| {
+                // This is not *really* the correct error, it will produce an
+                // IllegalArgumentException even though we're making a result. But also we shouldn't
+                // in practice try to return arrays of 2 billion IDs.
+                BridgeLayerError::IntegerOverflow(format!("{}_usize to i32", input.len()))
+            })?;
+            let array = env
+                .new_int_array(len)
+                .check_exceptions(env, "MismatchedDeviceError::convert_into")?;
+            let mut elems = unsafe { env.get_array_elements(&array, ReleaseMode::CopyBack) }
+                .check_exceptions(env, "MismatchedDeviceError::convert_into")?;
+
+            for (elem, id) in elems.iter_mut().zip(input) {
+                *elem = u8::from(*id).into();
+            }
+
+            // `elems` borrows from `array`, so we have to drop it before we return.
+            drop(elems);
+            Ok(array)
+        }
+
+        let account_bytes = account.convert_into(env)?;
+        let missing_devices = convert_device_list(env, missing_devices)?;
+        let extra_devices = convert_device_list(env, extra_devices)?;
+        let stale_devices = convert_device_list(env, stale_devices)?;
+
+        new_instance(
+            env,
+            ClassName("org.signal.libsignal.net.MismatchedDeviceException$Entry"),
+            jni_args!((
+                account_bytes => [byte],
+                missing_devices => [int],
+                extra_devices => [int],
+                stale_devices => [int]
+            ) -> void),
+        )
+    }
+}
+
+impl<'a> ResultTypeInfo<'a> for &'_ [libsignal_net_chat::api::messages::MismatchedDeviceError] {
+    type ResultType = JObjectArray<'a>;
+
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        let element_class = find_class(
+            env,
+            ClassName("org.signal.libsignal.net.MismatchedDeviceException$Entry"),
+        )
+        .check_exceptions(env, "MismatchedDeviceError::convert_into")?;
+        make_object_array(env, element_class, self)
+    }
+}
+
 impl<'a> ResultTypeInfo<'a> for libsignal_net_chat::api::registration::RegisterResponseBadge {
     type ResultType = JObject<'a>;
 
@@ -1705,11 +1927,12 @@ impl<'a> ResultTypeInfo<'a>
     type ResultType = JObjectArray<'a>;
 
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
-        make_object_array(
+        let element_class = find_class(
             env,
-            jni_class_name!(org.signal.libsignal.net.RegisterAccountResponse::BadgeEntitlement),
-            self,
+            ClassName("org.signal.libsignal.net.RegisterAccountResponse$BadgeEntitlement"),
         )
+        .check_exceptions(env, "RegisterResponseBadge::convert_into")?;
+        make_object_array(env, element_class, self)
     }
 }
 
@@ -1746,16 +1969,22 @@ impl<'a> ResultTypeInfo<'a>
     }
 }
 
+impl<'a> ResultTypeInfo<'a> for libsignal_net::chat::server_requests::DisconnectCause {
+    type ResultType = JThrowable<'a>;
+    const JNI_SIGNATURE: &'static str = jni_signature!(java.lang.Throwable);
+
+    fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
+        match self {
+            Self::LocalDisconnect => Ok(JObject::null().into()),
+            Self::Error(err) => SignalJniError::from(err).to_throwable(env),
+        }
+    }
+}
+
 /// Converts each element of `it` to a Java object, storing the result in an array.
-///
-/// `element_type_signature` should use [`jni_class_name`] if it's a plain class and
-/// [`jni_signature`] if it's an array (according to the official docs for the JNI [FindClass][]
-/// operation).
-///
-/// [FindClass]: https://docs.oracle.com/javase/8/docs/technotes/guides/jni/spec/functions.html#FindClass
 fn make_object_array<'a, It>(
     env: &mut JNIEnv<'a>,
-    element_type_signature: &str,
+    element_type: JClass<'a>,
     it: It,
 ) -> Result<JObjectArray<'a>, BridgeLayerError>
 where
@@ -1774,7 +2003,7 @@ where
                 // in practice try to return arrays of 2 billion objects.
                 BridgeLayerError::IntegerOverflow(format!("{len}_usize to i32"))
             })?,
-            element_type_signature,
+            element_type,
             JavaObject::null(),
         )
         .check_exceptions(env, "make_object_array")?;
@@ -1794,15 +2023,20 @@ where
 
 impl<'a> ResultTypeInfo<'a> for Box<[String]> {
     type ResultType = JObjectArray<'a>;
+    const JNI_SIGNATURE: &'static str = jni_signature!([java.lang.String]);
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
-        make_object_array(env, jni_class_name!(java.lang.String), self)
+        let element_class = find_class(env, ClassName("java.lang.String"))
+            .check_exceptions(env, "Box<[String]>::convert_into")?;
+        make_object_array(env, element_class, self)
     }
 }
 
 impl<'a> ResultTypeInfo<'a> for Box<[Vec<u8>]> {
     type ResultType = JObjectArray<'a>;
     fn convert_into(self, env: &mut JNIEnv<'a>) -> Result<Self::ResultType, BridgeLayerError> {
-        make_object_array(env, jni_signature!([byte]), self)
+        let element_class = find_primitive_array_class(env, jni_signature!([byte]))
+            .check_exceptions(env, "Box<[Vec<u8>]>::convert_into")?;
+        make_object_array(env, element_class, self)
     }
 }
 
@@ -1815,16 +2049,18 @@ impl<'a> ResultTypeInfo<'a> for MessageBackupValidationOutcome {
             found_unknown_fields,
         } = self;
 
+        let element_class = find_class(env, ClassName("java.lang.String"))
+            .check_exceptions(env, "MessageBackupValidationOutcome::convert_into")?;
         let unknown_fields = make_object_array(
             env,
-            jni_class_name!(java.lang.String),
+            element_class,
             found_unknown_fields.into_iter().map(|f| f.to_string()),
         )?;
         let error_message = error_message.convert_into(env)?;
 
         new_instance(
             env,
-            ClassName("org.signal.libsignal.protocol.util.Pair"),
+            ClassName("kotlin.Pair"),
             jni_args!((error_message => java.lang.Object, unknown_fields => java.lang.Object) -> void),
         )
     }
@@ -1932,25 +2168,18 @@ macro_rules! jni_bridge_handle_fns {
     };
 }
 
-macro_rules! trivial {
-    ($typ:ty) => {
-        impl SimpleArgTypeInfo<'_> for $typ {
-            type ArgType = Self;
-            fn convert_from(_env: &mut JNIEnv, foreign: &Self) -> Result<Self, BridgeLayerError> {
-                Ok(*foreign)
-            }
-        }
-        impl ResultTypeInfo<'_> for $typ {
-            type ResultType = Self;
-            fn convert_into(self, _env: &mut JNIEnv) -> Result<Self, BridgeLayerError> {
-                Ok(self)
-            }
-        }
-    };
+impl SimpleArgTypeInfo<'_> for i32 {
+    type ArgType = Self;
+    fn convert_from(_env: &mut JNIEnv, foreign: &Self) -> Result<Self, BridgeLayerError> {
+        Ok(*foreign)
+    }
+}
+impl ResultTypeInfo<'_> for i32 {
+    type ResultType = Self;
+    fn convert_into(self, _env: &mut JNIEnv) -> Result<Self, BridgeLayerError> {
+        Ok(self)
+    }
 }
-
-trivial!(i32);
-trivial!(());
 
 /// Syntactically translates `bridge_fn` argument types to JNI types for `cbindgen` and
 /// `gen_java_decl.py`.
@@ -2003,6 +2232,9 @@ macro_rules! jni_arg_type {
     (Box<dyn ChatListener >) =>{
         jni::JavaBridgeChatListener<'local>
     };
+    (Box<dyn ProvisioningListener >) =>{
+        jni::JavaBridgeProvisioningListener<'local>
+    };
     (Box<dyn ConnectChatBridge >) =>{
         $crate::jni::JavaConnectChatBridge<'local>
     };
@@ -2051,6 +2283,9 @@ macro_rules! jni_arg_type {
     (AccountEntropyPool) => {
         ::jni::objects::JString<'local>
     };
+    (MultiRecipientSendAuthorization) => {
+        $crate::jni::Nullable<::jni::objects::JByteArray<'local>>
+    };
     (ServiceIdSequence<'_>) => {
         ::jni::objects::JByteArray<'local>
     };
@@ -2102,6 +2337,13 @@ macro_rules! jni_arg_type {
     (TestingFutureCancellationGuard) => { ::jni::sys::jlong };
 
     (Ignored<$typ:ty>) => (::jni::objects::JObject<'local>);
+
+    // For use in callbacks.
+    (Result<$typ:tt $(, $ignored:ty)?>) => (jni_arg_type!($typ));
+    (Result<$typ:tt<$($args:tt),+> $(, $ignored:ty)?>) => (jni_arg_type!($typ<$($args),+>));
+    (Option<$typ:ty>) => ($crate::jni::Nullable<jni_arg_type!($typ)>);
+    (()) => (());
+    ($typ:ty) => (::jni::objects::JObject<'local>);
 }
 
 /// Syntactically translates `bridge_fn` result types to JNI types for `cbindgen` and
@@ -2156,8 +2398,8 @@ macro_rules! jni_result_type {
     (()) => {
         ()
     };
-    (($a:ty, $b:ty)) => {
-        $crate::jni::JavaPair<'local>
+    (($a:tt, $b:tt)) => {
+        $crate::jni::JavaPair<'local, $crate::jni_result_type!($a), $crate::jni_result_type!($b)>
     };
     (bool) => {
         ::jni::sys::jboolean
@@ -2197,6 +2439,9 @@ macro_rules! jni_result_type {
     (Vec<u8>) => {
         ::jni::objects::JByteArray<'local>
     };
+    (bytes::Bytes) => {
+        ::jni::objects::JByteArray<'local>
+    };
     (&[String]) => {
         ::jni::objects::JObjectArray<'local>
     };
@@ -2236,6 +2481,9 @@ macro_rules! jni_result_type {
     (CiphertextMessage) => {
         jni::JavaCiphertextMessage<'local>
     };
+    (Vec<ServiceId>) => {
+        ::jni::objects::JObjectArray<'local>
+    };
     (Box<[ChallengeOption] >) => {
         ::jni::objects::JObjectArray<'local>
     };
