@@ -4,9 +4,7 @@
 //
 
 mod curve25519;
-mod utils;
 
-use std::cmp::Ordering;
 use std::fmt;
 
 use curve25519_dalek::{MontgomeryPoint, scalar};
@@ -65,6 +63,17 @@ pub struct PublicKey {
     key: PublicKeyData,
 }
 
+// This implementation allows functions with the following signature
+// ```
+// fn foo(impl AsRef<PublicKey>) { ... }
+// ```
+// to accept both referenced `&PublicKey` and owned `PublicKey`.
+impl AsRef<PublicKey> for PublicKey {
+    fn as_ref(&self) -> &PublicKey {
+        self
+    }
+}
+
 impl PublicKey {
     fn new(key: PublicKeyData) -> Self {
         Self { key }
@@ -206,22 +215,6 @@ impl PartialEq for PublicKey {
     }
 }
 
-impl Ord for PublicKey {
-    fn cmp(&self, other: &Self) -> Ordering {
-        if self.key_type() != other.key_type() {
-            return self.key_type().cmp(&other.key_type());
-        }
-
-        utils::constant_time_cmp(self.key_data(), other.key_data())
-    }
-}
-
-impl PartialOrd for PublicKey {
-    fn partial_cmp(&self, other: &PublicKey) -> Option<Ordering> {
-        Some(self.cmp(other))
-    }
-}
-
 impl fmt::Debug for PublicKey {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
