@@ -19,7 +19,9 @@ use crate::backup::sticker::MessageStickerError;
 use crate::backup::time::{
     Duration, ReportUnusualTimestamp, Timestamp, TimestampError, TimestampOrForever,
 };
-use crate::backup::{BackupMeta, CallError, ReferencedTypes, TryIntoWith, likely_empty};
+use crate::backup::{
+    BackupMeta, CallError, HasUnknownFields, ReferencedTypes, TryIntoWith, likely_empty,
+};
 use crate::proto::backup as proto;
 
 mod contact_message;
@@ -66,6 +68,9 @@ use view_once_message::*;
 mod voice_message;
 use voice_message::*;
 
+mod pinned;
+use pinned::*;
+
 mod poll;
 use poll::*;
 
@@ -110,8 +115,8 @@ pub enum ChatItemError {
     OutgoingMessageFrom(RecipientId, DestinationKind),
     /// incoming message authored by contact {0:?} with no ACI or e164
     IncomingMessageFromContactWithoutAciOrE164(RecipientId),
-    /// ChatItem.item is a oneof but is empty
-    MissingItem,
+    /// ChatItem.item is a oneof but is empty with {0}
+    MissingItem(HasUnknownFields),
     /// StandardMessage has neither text nor attachments
     StandardMessageIsEmpty,
     /// text: {0}
@@ -128,14 +133,14 @@ pub enum ChatItemError {
     Reaction(#[from] ReactionError),
     /// payment: {0}
     Payment(#[from] PaymentError),
-    /// ChatUpdateMessage.update is a oneof but is empty
-    UpdateIsEmpty,
+    /// ChatUpdateMessage.update is a oneof but is empty with {0}
+    UpdateIsEmpty(HasUnknownFields),
     /// call error: {0}
     Call(#[from] CallError),
     /// GroupChange has no changes.
     GroupChangeIsEmpty,
-    /// for GroupUpdate change {0}, Update.update is a oneof but is empty
-    GroupChangeUpdateIsEmpty(usize),
+    /// for GroupUpdate change {0}, Update.update is a oneof but is empty with {1}
+    GroupChangeUpdateIsEmpty(usize, HasUnknownFields),
     /// group update: {0}
     GroupUpdate(#[from] GroupUpdateError),
     /// StickerMessage has no sticker
@@ -148,8 +153,8 @@ pub enum ChatItemError {
     ViewOnce(#[from] ViewOnceMessageError),
     /// direct story reply: {0}
     DirectStoryReply(#[from] DirectStoryReplyError),
-    /// ChatItem.directionalDetails is a oneof but is empty
-    NoDirection,
+    /// ChatItem.directionalDetails is a oneof but is empty with {0}
+    NoDirection(HasUnknownFields),
     /// directionless ChatItem wasn't an update message
     DirectionlessMessage,
     /// update message wasn't directionless
@@ -242,12 +247,18 @@ pub enum ChatItemError {
     InvalidTimestamp(#[from] TimestampError),
     /// invalid poll {0}
     InvalidPoll(#[from] PollError),
-    /// poll with a destination that is not group but {0:?}
-    PollNotInGroup(DestinationKind),
-    /// poll terminate with a destination that is not group but {0:?}
-    PollTerminateNotInGroup(DestinationKind),
+    /// unexpected poll destination {0:?}
+    PollUnexpectedDestination(DestinationKind),
+    /// unexpected poll terminate destination {0:?}
+    PollTerminateUnexpectedDestination(DestinationKind),
     /// poll terminate not from contact or self
     PollTerminateNotFromContact,
+    /// pin message not from contact or self
+    PinMessageNotFromContact,
+    /// pin message sent to release notes
+    PinMessageToReleaseNotes,
+    /// invalid pin message {0}
+    InvalidPinMessage(#[from] PinMessageError),
 }
 
 #[derive(Debug, thiserror::Error)]
@@ -358,6 +369,7 @@ pub struct ChatItemData<M: Method + ReferencedTypes> {
     /// The position of this chat item among all chat items (across chats) in
     /// the source stream.
     pub total_chat_item_order_index: usize,
+    pub pin_details: Option<PinDetails>,
     _limit_construction_to_module: (),
 }
 
@@ -553,6 +565,7 @@ impl<
         let proto::ChatItem {
             chatId: _,
             authorId,
+            pinDetails,
             item,
             directionalDetails,
             revisions,
@@ -560,11 +573,11 @@ impl<
             expiresInMs,
             dateSent,
             sms,
-            special_fields: _,
+            special_fields,
         } = self;
 
         let direction = directionalDetails
-            .ok_or(ChatItemError::NoDirection)?
+            .ok_or_else(|| ChatItemError::NoDirection(HasUnknownFields::check(&special_fields)))?
             .try_into_with(context)?;
 
         let author_id = RecipientId(authorId);
@@ -605,6 +618,7 @@ impl<
                     e164: None,
                     aci: None,
                     pni: _,
+                    username: _,
                 },
                 Direction::Incoming { .. },
             ) => Err(ChatItemError::IncomingMessageFromContactWithoutAciOrE164(
@@ -635,7 +649,7 @@ impl<
         }?;
 
         let message = item
-            .ok_or(ChatItemError::MissingItem)?
+            .ok_or_else(|| ChatItemError::MissingItem(HasUnknownFields::check(&special_fields)))?
             .try_into_with(context)?;
 
         let purpose = context.as_ref().purpose;
@@ -653,14 +667,12 @@ impl<
             update.validate_author(&cached_author_kind)?;
         }
 
-        if matches!(
-            (purpose, &message),
-            (
-                crate::backup::Purpose::TakeoutExport,
-                ChatItemMessage::ViewOnce(_)
-            )
-        ) {
-            return Err(InvalidExpiration::NotAllowedForPurpose(purpose).into());
+        if let (crate::backup::Purpose::TakeoutExport, ChatItemMessage::ViewOnce(view_once)) =
+            (purpose, &message)
+        {
+            if view_once.attachment.is_some() {
+                return Err(InvalidExpiration::NotAllowedForPurpose(purpose).into());
+            }
         }
 
         if !revisions.is_empty() {
@@ -806,6 +818,11 @@ impl<
             }
         }
 
+        let pin_details = pinDetails
+            .into_option()
+            .map(|x| x.try_into_with(context))
+            .transpose()?;
+
         Ok(ChatItemData {
             author: author.clone(),
             cached_author_kind,
@@ -816,6 +833,7 @@ impl<
             expire_start,
             expires_in,
             sms,
+            pin_details,
             total_chat_item_order_index: Default::default(),
             _limit_construction_to_module: (),
         })
@@ -888,8 +906,10 @@ impl<M: Method + ReferencedTypes> ChatItemData<M> {
                 }
             }
             ChatItemMessage::Poll(_) => {
-                if !recipient_data.is_group() {
-                    return Err(ChatItemError::PollNotInGroup((*recipient_data).into()));
+                if matches!(recipient_data, ChatRecipientKind::ReleaseNotes) {
+                    return Err(ChatItemError::PollUnexpectedDestination(
+                        (*recipient_data).into(),
+                    ));
                 }
             }
         }
@@ -1129,6 +1149,7 @@ mod test {
                 expireStartDate: Some(MillisecondsSinceEpoch::TEST_VALUE.0),
                 expiresInMs: Some(24 * 60 * 60 * 1000),
                 dateSent: MillisecondsSinceEpoch::TEST_VALUE.0,
+                pinDetails: Some(proto::chat_item::PinDetails::test_data()).into(),
                 ..Default::default()
             }
         }
@@ -1233,6 +1254,7 @@ mod test {
                 sent_at: Timestamp::test_value(),
                 sms: false,
                 total_chat_item_order_index: 0,
+                pin_details: Some(PinDetails::from_proto_test_data()),
                 _limit_construction_to_module: (),
             })
         )
@@ -1241,7 +1263,7 @@ mod test {
     #[test_case(|x| x.authorId = 0xffff => Err(ChatItemError::AuthorNotFound(RecipientId(0xffff))); "unknown_author")]
     #[test_case(|x| x.authorId = TestContext::GROUP_ID.0 => Err(ChatItemError::InvalidAuthor(TestContext::GROUP_ID, DestinationKind::Group)); "invalid author")]
     #[test_case(|x| x.authorId = TestContext::PNI_ONLY_ID.0 => Err(ChatItemError::IncomingMessageFromContactWithoutAciOrE164(TestContext::PNI_ONLY_ID)); "pni-only author")]
-    #[test_case(|x| x.directionalDetails = None => Err(ChatItemError::NoDirection); "no_direction")]
+    #[test_case(|x| x.directionalDetails = None => Err(ChatItemError::NoDirection(HasUnknownFields::No)); "no_direction")]
     #[test_case(|x| {
         x.authorId = TestContext::SELF_ID.0;
         x.directionalDetails = Some(proto::chat_item::OutgoingMessageDetails::test_data().into());
@@ -1505,7 +1527,7 @@ mod test {
     }
 
     #[test]
-    fn takeout_export_rejects_view_once_messages() {
+    fn takeout_export_rejects_view_once_messages_with_attachment() {
         let mut item = proto::ChatItem::test_data();
         item.expireStartDate = None;
         item.expiresInMs = None;
@@ -1529,7 +1551,7 @@ mod test {
         };
 
         let error = TryIntoWith::<ChatItemData<Store>, _>::try_into_with(item, &TestContext(meta))
-            .expect_err("view-once message should be rejected");
+            .expect_err("view-once message with attachment should be rejected");
 
         assert_matches!(
             error,
@@ -1539,6 +1561,34 @@ mod test {
         );
     }
 
+    #[test]
+    fn takeout_export_allows_view_once_messages_without_attachment() {
+        let mut item = proto::ChatItem::test_data();
+        item.expireStartDate = None;
+        item.expiresInMs = None;
+        item.item = Some(proto::chat_item::Item::ViewOnceMessage(
+            proto::ViewOnceMessage {
+                attachment: None.into(),
+                reactions: vec![proto::Reaction::test_data()],
+                ..Default::default()
+            },
+        ));
+
+        let meta = BackupMeta {
+            purpose: Purpose::TakeoutExport,
+            backup_time: Timestamp::test_value(),
+            media_root_backup_key: libsignal_account_keys::BackupKey(
+                [0xab; libsignal_account_keys::BACKUP_KEY_LEN],
+            ),
+            version: 0,
+            current_app_version: "libsignal-testing 0.0.2".into(),
+            first_app_version: "libsignal-testing 0.0.1".into(),
+        };
+
+        let result = TryIntoWith::<ChatItemData<Store>, _>::try_into_with(item, &TestContext(meta));
+        assert_matches!(result, Ok(_));
+    }
+
     #[test]
     fn expiration_start_without_duration() {
         let mut item = proto::ChatItem::test_data();
@@ -1700,6 +1750,27 @@ mod test {
         Err(ChatItemError::DirectStoryReplyNotInContactThread(DestinationKind::Group));
         "direct story reply in group chat"
     )]
+    #[test_case(
+        TestContext::RELEASE_NOTES_ID,
+        |x| {
+            x.authorId = TestContext::RELEASE_NOTES_ID.0;
+            x.item = Some(proto::chat_item::Item::Poll(proto::Poll::test_data()))
+        } => Err(ChatItemError::PollUnexpectedDestination(DestinationKind::ReleaseNotes));
+        "poll from release notes"
+    )]
+    #[test_case(
+        TestContext::RELEASE_NOTES_ID,
+        |x| {
+            x.directionalDetails = Some(
+                proto::chat_item::DirectionalDetails::Directionless(
+                    proto::chat_item::DirectionlessMessageDetails::default()));
+            x.authorId = TestContext::SELF_ID.0;
+            x.item = Some(proto::chat_item::Item::UpdateMessage(proto::ChatUpdateMessage {
+                update: Some(proto::chat_update_message::Update::PollTerminate(proto::PollTerminateUpdate::test_data())),
+                ..proto::ChatUpdateMessage::default()
+            }))
+        } => Err(ChatItemError::PollTerminateUnexpectedDestination(DestinationKind::ReleaseNotes));
+        "poll terminate update to release notes")]
     fn validate_chat_recipient(
         recipient_id: RecipientId,
         modifier: fn(&mut proto::ChatItem),
