@@ -2,15 +2,16 @@
 // SPDX-License-Identifier: AGPL-3.0-only
 //
 
-use crate::backup::TryIntoWith;
 use crate::backup::call::{GroupCall, IndividualCall};
 use crate::backup::chat::ChatItemError;
 use crate::backup::chat::group::GroupChatUpdate;
+use crate::backup::chat::pinned::PinMessageUpdate;
 use crate::backup::chat::poll::PollTerminate;
 use crate::backup::frame::RecipientId;
 use crate::backup::method::LookupPair;
 use crate::backup::recipient::{ChatItemAuthorKind, ChatRecipientKind, E164, MinimalRecipientData};
 use crate::backup::time::{Duration, ReportUnusualTimestamp};
+use crate::backup::{HasUnknownFields, TryIntoWith};
 use crate::proto::backup as proto;
 
 /// Validated version of [`proto::chat_update_message::Update`].
@@ -27,6 +28,7 @@ pub enum UpdateMessage<Recipient> {
     GroupCall(GroupCall<Recipient>),
     LearnedProfileUpdate(proto::learned_profile_chat_update::PreviousName),
     PollTerminate(PollTerminate),
+    PinMessage(PinMessageUpdate<Recipient>),
 }
 
 /// Validated version of [`proto::simple_chat_update::Type`].
@@ -59,10 +61,12 @@ impl<C: LookupPair<RecipientId, MinimalRecipientData, R> + ReportUnusualTimestam
     fn try_into_with(self, context: &C) -> Result<UpdateMessage<R>, Self::Error> {
         let proto::ChatUpdateMessage {
             update,
-            special_fields: _,
+            special_fields,
         } = self;
 
-        let update = update.ok_or(ChatItemError::UpdateIsEmpty)?;
+        let update = update.ok_or_else(|| {
+            ChatItemError::UpdateIsEmpty(HasUnknownFields::check(&special_fields))
+        })?;
 
         use proto::chat_update_message::Update;
         Ok(match update {
@@ -111,11 +115,15 @@ impl<C: LookupPair<RecipientId, MinimalRecipientData, R> + ReportUnusualTimestam
                                 i,
                                 proto::group_change_chat_update::Update {
                                     update,
-                                    special_fields: _,
+                                    special_fields,
                                 },
                             )| {
-                                let update =
-                                    update.ok_or(ChatItemError::GroupChangeUpdateIsEmpty(i))?;
+                                let update = update.ok_or_else(|| {
+                                    ChatItemError::GroupChangeUpdateIsEmpty(
+                                        i,
+                                        HasUnknownFields::check(&special_fields),
+                                    )
+                                })?;
                                 GroupChatUpdate::try_from(update).map_err(ChatItemError::from)
                             },
                         )
@@ -170,6 +178,7 @@ impl<C: LookupPair<RecipientId, MinimalRecipientData, R> + ReportUnusualTimestam
             Update::PollTerminate(proto) => {
                 UpdateMessage::PollTerminate(proto.try_into_with(context)?)
             }
+            Update::PinMessage(proto) => UpdateMessage::PinMessage(proto.try_into_with(context)?),
         })
     }
 }
@@ -301,6 +310,13 @@ impl<R> UpdateMessage<R> {
                     Ok(())
                 }
             }
+            UpdateMessage::PinMessage(_) => {
+                if !author.is_valid_sender_account() {
+                    Err(ChatItemError::PinMessageNotFromContact)
+                } else {
+                    Ok(())
+                }
+            }
         }
     }
 
@@ -335,8 +351,6 @@ impl<R> UpdateMessage<R> {
                 | SimpleChatUpdate::BadDecrypt
                 | SimpleChatUpdate::UnsupportedProtocolMessage
                 | SimpleChatUpdate::ReportedSpam
-                | SimpleChatUpdate::Blocked
-                | SimpleChatUpdate::Unblocked
                 | SimpleChatUpdate::MessageRequestAccepted,
             )
             | UpdateMessage::ProfileChange { .. }
@@ -360,6 +374,11 @@ impl<R> UpdateMessage<R> {
                     Ok(())
                 }
             }
+            UpdateMessage::Simple(SimpleChatUpdate::Blocked | SimpleChatUpdate::Unblocked) => {
+                // It is possible to block the Release Notes chat in addition to a contact or group.
+                // Self is allowed because of possible past thread merging; see above.
+                Ok(())
+            }
             UpdateMessage::GroupChange { .. } => {
                 if !matches!(chat, ChatRecipientKind::Group) {
                     Err(ChatItemError::GroupUpdateNotInGroupThread)
@@ -403,8 +422,17 @@ impl<R> UpdateMessage<R> {
                 }
             }
             UpdateMessage::PollTerminate(_) => {
-                if !chat.is_group() {
-                    Err(ChatItemError::PollTerminateNotInGroup((*chat).into()))
+                if matches!(chat, ChatRecipientKind::ReleaseNotes) {
+                    Err(ChatItemError::PollTerminateUnexpectedDestination(
+                        (*chat).into(),
+                    ))
+                } else {
+                    Ok(())
+                }
+            }
+            UpdateMessage::PinMessage(_) => {
+                if matches!(chat, ChatRecipientKind::ReleaseNotes) {
+                    Err(ChatItemError::PinMessageToReleaseNotes)
                 } else {
                     Ok(())
                 }
@@ -416,10 +444,11 @@ impl<R> UpdateMessage<R> {
 #[cfg(test)]
 mod test {
     use assert_matches::assert_matches;
-    use test_case::test_case;
+    use test_case::{test_case, test_matrix};
 
     use super::*;
     use crate::backup::call::CallError;
+    use crate::backup::chat::pinned::PinMessageError;
     use crate::backup::testutil::TestContext;
     use crate::proto::backup::chat_update_message::Update as ChatUpdateProto;
 
@@ -478,7 +507,7 @@ mod test {
     fn chat_update_message_no_item() {
         assert_matches!(
             proto::ChatUpdateMessage::default().try_into_with(&TestContext::default()),
-            Err(ChatItemError::UpdateIsEmpty)
+            Err(ChatItemError::UpdateIsEmpty(HasUnknownFields::No))
         );
     }
 
@@ -522,6 +551,18 @@ mod test {
         proto::LearnedProfileChatUpdate::default(),
         Err(ChatItemError::LearnedProfileIsEmpty)
     )]
+    #[test_case(
+        proto::PinMessageUpdate::default(),
+        Err(ChatItemError::InvalidPinMessage(PinMessageError::UnknownAuthorId))
+    )]
+    #[test_case(
+        proto::PinMessageUpdate::test_data(),
+        Ok(())
+    )]
+    #[test_case(
+        proto::PollTerminateUpdate::test_data(),
+        Ok(())
+    )]
     fn chat_update_message_item(
         update: impl Into<proto::chat_update_message::Update>,
         expected: Result<(), ChatItemError>,
@@ -560,4 +601,33 @@ mod test {
 
         assert_eq!(result, expected);
     }
+
+    #[test_matrix(
+        [proto::simple_chat_update::Type::BLOCKED, proto::simple_chat_update::Type::UNBLOCKED],
+        [
+            ChatRecipientKind::Self_,
+            ChatRecipientKind::Group,
+            ChatRecipientKind::Contact { has_aci: true },
+            ChatRecipientKind::Contact { has_aci: false },
+            ChatRecipientKind::ReleaseNotes
+        ]
+    )]
+    fn updates_for_every_chat(
+        update_type: proto::simple_chat_update::Type,
+        chat_kind: ChatRecipientKind,
+    ) {
+        let um: UpdateMessage<_> = proto::ChatUpdateMessage {
+            update: Some(proto::chat_update_message::Update::SimpleUpdate(
+                proto::SimpleChatUpdate {
+                    type_: update_type.into(),
+                    special_fields: Default::default(),
+                },
+            )),
+            ..Default::default()
+        }
+        .try_into_with(&TestContext::default())
+        .expect("Conversion should succeed for a valid update");
+        um.validate_chat_recipient(&chat_kind)
+            .expect("should be valid");
+    }
 }
