@@ -21,12 +21,14 @@ use base64::prelude::BASE64_STANDARD;
 use http::StatusCode;
 use libsignal_net::chat;
 use libsignal_net::infra::errors::LogSafeDisplay;
+use libsignal_net::infra::http_client::Http2Client;
 use libsignal_net::infra::{AsHttpHeader, extract_retry_later};
 use serde_with::serde_as;
 
 use crate::api::{
     ChallengeOption, DisconnectedError, RateLimitChallenge, RequestError, UserBasedAuthorization,
 };
+use crate::logging::DebugAsStrOrBytes;
 
 const ACCESS_KEY_HEADER_NAME: http::HeaderName =
     http::HeaderName::from_static("unidentified-access-key");
@@ -51,6 +53,9 @@ impl AsHttpHeader for UserBasedAuthorization {
     }
 }
 
+/// Marker type for use in [`crate::api`] traits.
+pub enum OverWs {}
+
 /// An abstraction over [`chat::ChatConnection`].
 pub trait WsConnection: Sync {
     fn send(
@@ -59,6 +64,13 @@ pub trait WsConnection: Sync {
         log_safe_path: &str,
         request: chat::Request,
     ) -> impl Future<Output = Result<chat::Response, chat::SendError>> + Send;
+
+    fn grpc_service_to_use_instead(
+        &self,
+        _message: &'static str,
+    ) -> impl Future<Output = Option<impl crate::grpc::GrpcServiceProvider>> + Send {
+        std::future::ready(None::<Http2Client<chat::GrpcBody>>)
+    }
 }
 
 impl WsConnection for chat::ChatConnection {
@@ -103,6 +115,22 @@ impl WsConnection for chat::ChatConnection {
 
         result
     }
+
+    fn grpc_service_to_use_instead(
+        &self,
+        message: &'static str,
+    ) -> impl Future<Output = Option<impl crate::grpc::GrpcServiceProvider>> + Send {
+        use futures_util::future::Either;
+
+        match self
+            .grpc_overrides()
+            .get(message)
+            .unwrap_or(&chat::GrpcOverride::UseWs)
+        {
+            chat::GrpcOverride::UseGrpc => Either::Left(self.shared_h2_connection()),
+            chat::GrpcOverride::UseWs => Either::Right(std::future::ready(None)),
+        }
+    }
 }
 
 impl<E> From<chat::SendError> for RequestError<E> {
@@ -324,16 +352,6 @@ where
     })
 }
 
-struct DebugAsStrOrBytes<'b>(&'b [u8]);
-impl std::fmt::Debug for DebugAsStrOrBytes<'_> {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match std::str::from_utf8(self.0) {
-            Ok(s) => s.fmt(f),
-            Err(_) => hex::encode(self.0).fmt(f),
-        }
-    }
-}
-
 #[cfg(test)]
 mod testutil {
     use super::*;
