@@ -12,7 +12,7 @@ use base64::prelude::{
 use http::header::ACCEPT;
 use http::uri::PathAndQuery;
 use libsignal_core::{Aci, E164};
-use libsignal_keytrans::{AccountData, LastTreeHead};
+use libsignal_keytrans::{AccountData, LastTreeHead, Versioned};
 use libsignal_net::chat;
 use libsignal_protocol::PublicKey;
 use serde::{Deserialize, Serialize};
@@ -20,6 +20,7 @@ use serde::{Deserialize, Serialize};
 use super::{CONTENT_TYPE_JSON, CustomError, TryIntoResponse as _, WsConnection};
 use crate::api::keytrans::*;
 use crate::api::{RequestError, Unauth};
+use crate::logging::DebugAsStrOrBytes;
 
 const SEARCH_PATH: &str = "/v1/key-transparency/search";
 const DISTINGUISHED_PATH: &str = "/v1/key-transparency/distinguished";
@@ -66,25 +67,34 @@ struct RawChatSearchRequest {
     #[serde(skip_serializing_if = "Option::is_none")]
     last_tree_head_size: Option<u64>,
     distinguished_tree_head_size: u64,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    aci_version: Option<u32>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    e164_version: Option<u32>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    username_hash_version: Option<u32>,
 }
 
 impl RawChatSearchRequest {
     fn new(
-        aci: &Aci,
+        aci: Versioned<&Aci>,
         aci_identity_key: &PublicKey,
-        e164: Option<&(E164, Vec<u8>)>,
-        username_hash: Option<&UsernameHash>,
+        e164: Option<Versioned<&(E164, Vec<u8>)>>,
+        username_hash: Option<Versioned<&UsernameHash>>,
         last_tree_head_size: Option<u64>,
         distinguished_tree_head_size: u64,
     ) -> Self {
         Self {
-            aci: aci.as_chat_value(),
+            aci: aci.item.as_chat_value(),
             aci_identity_key: BASE64_STANDARD.encode(aci_identity_key.serialize()),
-            e164: e164.map(|x| x.0.as_chat_value()),
-            username_hash: username_hash.map(|x| x.as_chat_value()),
-            unidentified_access_key: e164.map(|x| BASE64_STANDARD.encode(&x.1)),
+            e164: e164.as_ref().map(|x| x.item.0.as_chat_value()),
+            username_hash: username_hash.as_ref().map(|x| x.item.as_chat_value()),
+            unidentified_access_key: e164.as_ref().map(|x| BASE64_STANDARD.encode(&x.item.1)),
             last_tree_head_size,
             distinguished_tree_head_size,
+            aci_version: aci.version,
+            e164_version: e164.and_then(|x| x.version),
+            username_hash_version: username_hash.and_then(|x| x.version),
         }
     }
 }
@@ -93,7 +103,11 @@ impl From<RawChatSearchRequest> for chat::Request {
     fn from(request: RawChatSearchRequest) -> Self {
         Self {
             method: http::Method::POST,
-            body: Some(serde_json::to_vec(&request).unwrap().into()),
+            body: Some(
+                serde_json::to_vec(&request)
+                    .expect("can convert to JSON")
+                    .into(),
+            ),
             headers: common_headers(),
             path: PathAndQuery::from_static(SEARCH_PATH),
         }
@@ -181,7 +195,11 @@ impl From<RawChatMonitorRequest> for chat::Request {
     fn from(request: RawChatMonitorRequest) -> Self {
         Self {
             method: http::Method::POST,
-            body: Some(serde_json::to_vec(&request).unwrap().into()),
+            body: Some(
+                serde_json::to_vec(&request)
+                    .expect("can convert to JSON")
+                    .into(),
+            ),
             headers: common_headers(),
             path: PathAndQuery::from_static(MONITOR_PATH),
         }
@@ -215,8 +233,12 @@ impl RawChatMonitorRequest {
             e164: e164.map(|e164| {
                 ValueMonitor::for_e164(
                     e164,
-                    account_data.e164.as_ref().unwrap().latest_log_position(),
-                    &account_data.e164.as_ref().unwrap().index,
+                    account_data
+                        .e164
+                        .as_ref()
+                        .expect("checked above")
+                        .latest_log_position(),
+                    &account_data.e164.as_ref().expect("checked above").index,
                 )
             }),
             username_hash: username_hash.as_ref().map(|unh| {
@@ -225,9 +247,13 @@ impl RawChatMonitorRequest {
                     account_data
                         .username_hash
                         .as_ref()
-                        .unwrap()
+                        .expect("checked above")
                         .latest_log_position(),
-                    &account_data.username_hash.as_ref().unwrap().index,
+                    &account_data
+                        .username_hash
+                        .as_ref()
+                        .expect("checked above")
+                        .index,
                 )
             }),
             last_non_distinguished_tree_head_size,
@@ -243,8 +269,8 @@ impl<T: WsConnection> Unauth<T> {
     ) -> Result<Vec<u8>, RequestError<Error>> {
         log::debug!("{}", &request.path.as_str());
         log::debug!(
-            "{}",
-            &String::from_utf8(request.clone().body.unwrap_or_default().to_vec()).unwrap()
+            "{:?}",
+            DebugAsStrOrBytes(request.body.as_deref().unwrap_or_default())
         );
         // All KT requests keep identifying information out of the path+query, so it's okay to log
         // it. The Distinguished request does include the tree size, but that only reveals when the
@@ -276,10 +302,10 @@ impl<T: WsConnection> Unauth<T> {
 impl<T: WsConnection> LowLevelChatApi for Unauth<T> {
     async fn search(
         &self,
-        aci: &Aci,
+        aci: Versioned<&Aci>,
         aci_identity_key: &PublicKey,
-        e164: Option<&(E164, Vec<u8>)>,
-        username_hash: Option<&UsernameHash<'_>>,
+        e164: Option<Versioned<&(E164, Vec<u8>)>>,
+        username_hash: Option<Versioned<&UsernameHash<'_>>>,
         stored_account_data: Option<&AccountData>,
         distinguished_tree_head: &LastTreeHead,
     ) -> Result<Vec<u8>, RequestError<Error>> {
@@ -419,10 +445,10 @@ mod test_support {
 
         let account_data = kt
             .search(
-                &aci,
+                (&aci).into(),
                 &aci_identity_key,
-                Some(e164.clone()),
-                Some(username_hash.clone()),
+                Some(e164.clone().into()),
+                Some(username_hash.clone().into()),
                 None,
                 &distinguished_tree,
             )
@@ -447,10 +473,10 @@ mod test_support {
         prompt("Now advance the tree. Yes, again! (and press ENTER)");
 
         let raw_request = RawChatSearchRequest::new(
-            &aci,
+            (&aci).into(),
             &aci_identity_key,
-            Some(&e164),
-            Some(&username_hash),
+            Some((&e164).into()),
+            Some((&username_hash).into()),
             Some(account_data.last_tree_head.0.tree_size),
             distinguished_tree.0.tree_size,
         );
@@ -470,7 +496,7 @@ mod test_support {
         }
 
         println!(
-            "const CHAT_SEARCH_RESPONSE_VALID_AT: Duration = Duration::from_secs({});",
+            "Update the net/tests/data/chat_response_valid_at.in file to: `Duration::from_secs({})`",
             SystemTime::UNIX_EPOCH.elapsed().unwrap().as_secs()
         );
 
@@ -486,6 +512,9 @@ mod test {
 
     use assert_matches::assert_matches;
     use libsignal_keytrans::LocalStateUpdate;
+    use libsignal_protocol::IdentityKeyPair;
+    use rand::TryRngCore;
+    use rand::rngs::OsRng;
     use test_case::test_case;
 
     use super::test_support::{
@@ -527,10 +556,10 @@ mod test {
                 let known_account_data = test_account_data();
 
                 kt.search(
-                    &aci,
+                    (&aci).into(),
                     &aci_identity_key,
-                    use_e164.then_some(e164),
-                    use_username_hash.then_some(username_hash),
+                    use_e164.then_some(e164.into()),
+                    use_username_hash.then_some(username_hash.into()),
                     Some(known_account_data),
                     &test_distinguished_tree(),
                 )
@@ -542,6 +571,38 @@ mod test {
         .expect("can search");
     }
 
+    #[tokio::test]
+    async fn search_with_version_integration_test() {
+        if !kt_integration_enabled() {
+            println!("SKIPPED: running integration tests is not enabled");
+            return;
+        }
+        retry_n(
+            NETWORK_RETRY_COUNT,
+            || async {
+                let chat = make_chat().await;
+                let kt = make_kt(&chat);
+
+                let known_account_data = test_account_data();
+                let aci_version = known_account_data.aci.greatest_version();
+
+                kt.search(
+                    Versioned::new(&test_account::aci(), aci_version),
+                    // (&test_account::aci()).into(),
+                    &test_account::aci_identity_key(),
+                    Some(test_account::e164_pair().into()),
+                    Some(test_account::username_hash().into()),
+                    None,
+                    &test_distinguished_tree(),
+                )
+                .await
+            },
+            should_retry,
+        )
+        .await
+        .expect("can search with version");
+    }
+
     #[tokio::test]
     #[test_case(false; "unknown_distinguished")]
     #[test_case(true; "known_distinguished")]
@@ -626,7 +687,7 @@ mod test {
     }
 
     #[tokio::test]
-    async fn search_for_deleted_account() {
+    async fn search_with_wrong_identity_key() {
         if !kt_integration_enabled() {
             println!("SKIPPED: running integration tests is not enabled");
             return;
@@ -635,15 +696,15 @@ mod test {
         let chat = make_chat().await;
         let kt = make_kt(&chat);
 
-        // This ACI belongs to account 18005550102
-        // The correct ACI identity key is `hex!("05b65b151f64638b0ea549efb2989e9d726ad2b87fbca1328d872ed6f8fbb7a333")`
-        let aci = Aci::from(uuid::uuid!("4129e9d6-dbb3-4f44-97b4-2dd29f0e2681"));
-
-        let wrong_identity_key = test_account::aci_identity_key();
+        let wrong_identity_key = {
+            let mut rng = OsRng.unwrap_err();
+            let key_pair = IdentityKeyPair::generate(&mut rng);
+            *key_pair.public_key()
+        };
 
         let result = kt
             .search(
-                &aci,
+                (&test_account::aci()).into(),
                 &wrong_identity_key,
                 None,
                 None,
@@ -673,7 +734,7 @@ mod test {
 
         let result = kt
             .search(
-                &aci,
+                (&aci).into(),
                 &wrong_identity_key,
                 None,
                 None,
@@ -683,7 +744,7 @@ mod test {
             .await;
         assert_matches!(
             result,
-            Err(RequestError::Unexpected { log_safe: msg }) if msg == "unexpected response status 404 Not Found"
+            Err(RequestError::Unexpected { log_safe: msg }) if msg == "unexpected response status 403 Forbidden"
         );
     }
 }
