@@ -17,6 +17,7 @@ use thiserror::Error;
 use tungstenite::protocol::CloseFrame;
 use tungstenite::protocol::frame::coding::CloseCode;
 use uuid::Uuid;
+use zerocopy::{FromBytes, Immutable, KnownLayout};
 
 use crate::auth::Auth;
 use crate::connect_state::{ConnectionResources, WebSocketTransportConnectorFactory};
@@ -141,19 +142,17 @@ impl TryFrom<ClientResponse> for LookupResponse {
             debug_permits_used,
         } = response;
 
-        if e164_pni_aci_triples.len() % LookupResponseEntry::SERIALIZED_LEN != 0 {
+        let (record_chunks, record_remainder) =
+            e164_pni_aci_triples.as_chunks::<{ LookupResponseEntry::SERIALIZED_LEN }>();
+
+        if !record_remainder.is_empty() {
             return Err(CdsiProtocolError::InvalidNumberOfBytes {
                 actual_length: e164_pni_aci_triples.len(),
             });
         }
-
-        let records = e164_pni_aci_triples
-            .chunks(LookupResponseEntry::SERIALIZED_LEN)
-            .flat_map(|record| {
-                LookupResponseEntry::try_parse_from(
-                    record.try_into().expect("chunk size is correct"),
-                )
-            })
+        let records = record_chunks
+            .iter()
+            .flat_map(LookupResponseEntry::try_parse_from)
             .collect();
 
         Ok(Self {
@@ -165,20 +164,29 @@ impl TryFrom<ClientResponse> for LookupResponse {
 
 impl LookupResponseEntry {
     fn try_parse_from(record: &[u8; Self::SERIALIZED_LEN]) -> Option<Self> {
-        fn non_nil_uuid<T: From<Uuid>>(bytes: &uuid::Bytes) -> Option<T> {
-            let uuid = Uuid::from_bytes(*bytes);
+        fn non_nil_uuid<T: From<Uuid>>(bytes: uuid::Bytes) -> Option<T> {
+            let uuid = Uuid::from_bytes(bytes);
             (!uuid.is_nil()).then(|| uuid.into())
         }
 
-        // TODO(https://github.com/rust-lang/rust/issues/90091): use split_array
-        // instead of expect() on the output.
-        let (e164_bytes, record) = record.split_at(E164::SERIALIZED_LEN);
-        let e164_bytes = <&[u8; E164::SERIALIZED_LEN]>::try_from(e164_bytes).expect("split at len");
-        let e164 = E164::from_be_bytes(*e164_bytes)?;
-        let (pni_bytes, aci_bytes) = record.split_at(Uuid::SERIALIZED_LEN);
+        // Decode record into its component parts.
+        #[derive(FromBytes, Immutable, KnownLayout)]
+        #[repr(C)]
+        struct RecordRepr {
+            e164_bytes: [u8; E164::SERIALIZED_LEN],
+            pni_bytes: [u8; Uuid::SERIALIZED_LEN],
+            aci_bytes: [u8; Uuid::SERIALIZED_LEN],
+        }
 
-        let pni = non_nil_uuid(pni_bytes.try_into().expect("split at len"));
-        let aci = non_nil_uuid(aci_bytes.try_into().expect("split at len"));
+        let RecordRepr {
+            e164_bytes,
+            pni_bytes,
+            aci_bytes,
+        } = zerocopy::transmute_ref!(record);
+
+        let e164 = E164::from_be_bytes(*e164_bytes)?;
+        let pni = non_nil_uuid(*pni_bytes);
+        let aci = non_nil_uuid(*aci_bytes);
 
         Some(Self { e164, aci, pni })
     }
@@ -482,7 +490,6 @@ mod test {
         AsStaticHttpHeader as _, EnableDomainFronting, RECOMMENDED_WS_CONFIG,
     };
     use nonzero_ext::nonzero;
-    use tokio_stream::wrappers::UnboundedReceiverStream;
     use tungstenite::protocol::CloseFrame;
     use tungstenite::protocol::frame::coding::CloseCode;
     use uuid::Uuid;
@@ -491,6 +498,7 @@ mod test {
     use super::*;
     use crate::auth::Auth;
     use crate::connect_state::{ConnectState, SUGGESTED_CONNECT_CONFIG};
+    use crate::env::StaticIpOrder;
 
     #[test]
     fn parse_lookup_response_entries() {
@@ -920,23 +928,25 @@ mod test {
 
     #[test_log::test(tokio::test)]
     async fn websocket_rejected_with_http_429_too_many_requests() {
-        let h2_server = warp::get().then(|| async move {
+        let service = warp::get().then(|| async move {
             let reply = warp::reply();
             let reply = warp::reply::with_header(reply, RetryLater::HEADER_NAME.as_str(), "100");
             warp::reply::with_status(reply, warp::http::StatusCode::TOO_MANY_REQUESTS)
         });
 
-        let (tx_connections, incoming_connections) = tokio::sync::mpsc::unbounded_channel();
-        tokio::spawn(
-            warp::serve(h2_server)
-                .serve_incoming(UnboundedReceiverStream::new(incoming_connections)),
-        );
+        let (tx_connections, mut incoming_connections) = tokio::sync::mpsc::unbounded_channel();
+        tokio::spawn(async move {
+            while let Some(conn) = incoming_connections.recv().await {
+                tokio::spawn(hyper::server::conn::http1::Builder::new().serve_connection(
+                    hyper_util::rt::TokioIo::new(conn),
+                    hyper_util::service::TowerToHyperService::new(warp::service(service)),
+                ));
+            }
+        });
 
         let connector = ConnectFn(|(), _route| {
             let (local, remote) = tokio::io::duplex(1024);
-            tx_connections
-                .send(Ok::<_, TransportConnectError>(local))
-                .unwrap();
+            tx_connections.send(local).unwrap();
             std::future::ready(Ok::<_, TransportConnectError>(remote))
         });
 
@@ -952,7 +962,10 @@ mod test {
         let network_change_event = no_network_change_events();
 
         // If we don't mock out the DNS, this test will fail on machines without internet access.
-        let static_map = HashMap::from([env.cdsi.domain_config.static_fallback()]);
+        let static_map = HashMap::from([env
+            .cdsi
+            .domain_config
+            .static_fallback(StaticIpOrder::HARDCODED)]);
         let dns_resolver = DnsResolver::new_from_static_map(static_map);
 
         let result = CdsiConnection::connect_with(
